@@ -2,7 +2,7 @@
 
 """
 Usage: process_mails FILE [--requeue_errors] [--list_emails=<number>] [--get_eml=<mail_id>] [--gen_pdf=<mail_id>]
-                          [--get_eml_orig] [--stats]
+                          [--get_eml_orig] [--reset_flags=<mail_id>] [--test_eml=<path>] [--stats]
 
 Arguments:
     FILE         config file
@@ -14,8 +14,11 @@ Options:
     --get_eml=<mail_id>     Get eml of original/contained email id.
     --get_eml_orig          Get eml of original email id (otherwise contained).
     --gen_pdf=<mail_id>     Generate pdf of contained email id.
+    --reset_flags=<mail_id> Reset all flags of email id
+    --test_eml=<path>       Test an eml handling
     --stats                 Get email stats following stats
 """
+
 from datetime import datetime
 from datetime import timedelta
 from docopt import docopt
@@ -24,6 +27,7 @@ from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
 from hashlib import md5
 from imio.email.dms import dev_mode
+from imio.email.dms import email_policy
 from imio.email.dms import logger
 from imio.email.dms.imap import IMAPEmailHandler
 from imio.email.dms.imap import MailData
@@ -33,11 +37,15 @@ from imio.email.dms.utils import safe_unicode
 from imio.email.dms.utils import save_as_eml
 from imio.email.dms.utils import set_next_id
 from imio.email.parser.parser import Parser  # noqa
+from imio.email.parser.utils import stop  # noqa
+from imio.email.parser.utils import structure  # noqa
 from io import BytesIO
 from PIL import Image
 from PIL import UnidentifiedImageError
 from smtplib import SMTP
+
 import configparser
+import email
 import imaplib
 import json
 import os
@@ -56,6 +64,8 @@ except ImportError:
 
 dev_infos = {'nid': None}
 img_size_limit = 1024
+# originally 89478485 => blocks at > 13300 pixels square
+Image.MAX_IMAGE_PIXELS = None
 
 ERROR_MAIL = u"""
 Problematic mail is attached.\n
@@ -252,7 +262,10 @@ def get_mailbox_infos(config):
 def get_preview_pdf_path(config, mail_id):
     mail_infos = config["mailinfos"]
     output_dir = mail_infos["pdf-output-dir"]
-    filename = "{0}.pdf".format(mail_id.decode("UTF-8"))
+    if isinstance(mail_id, bytes):
+        filename = "{0}.pdf".format(mail_id.decode("UTF-8"))
+    else:
+        filename = "{0}.pdf".format(mail_id)
     return os.path.join(output_dir, filename)
 
 
@@ -271,6 +284,8 @@ def modify_attachments(mail_id, attachments):
             except UnidentifiedImageError as msg:
                 new_lst.append(dic)  # kept original image
                 continue
+            except Image.DecompressionBombError as msg:  # never append because Image.MAX_IMAGE_PIXELS is set to None
+                continue
             is_reduced, new_size = get_reduced_size(img.size, img_size_limit)
             new_img = img
             if is_reduced:
@@ -324,8 +339,9 @@ def send_to_ws(config, headers, main_file_path, attachments, mail_id):
             attachment_info = tarfile.TarInfo(name='/attachments/{}'.format(attachment['filename']))
             attachment_info.size = len(attachment_contents)
             tar.addfile(tarinfo=attachment_info, fileobj=BytesIO(attachment_contents))
-
-    if not dev_mode:  # we send to the ws
+    if dev_mode:
+        logger.info("tar file '{}' created".format(tar_path))
+    else:  # we send to the ws
         tar_content = tar_path.read_bytes()
         now = datetime.now()
         metadata = {
@@ -405,25 +421,26 @@ def process_mails():
         # import ipdb; ipdb.set_trace()
         # handler.mark_reset_error('58')
         # handler.mark_reset_ignored('77')
-        # res, data = handler.connection.search(None, 'SUBJECT "JBC client"')
+        # res, data = handler.connection.search(None, 'SUBJECT "FAIGNART MARION"')
         # for mail_id in data[0].split():
         #      omail = handler.get_mail(mail_id)
         #      parser = Parser(omail, dev_mode, mail_id)
         #      headers = parser.headers
         #      amail = parser.message
         #      parsed = MailParser(omail)
-        #     logger.info(email['Subject'])
+        #     logger.info(headers['Subject'])
         handler.disconnect()
         lock.close()
         sys.exit()
     elif arguments.get("--get_eml"):
         mail_id = arguments['--get_eml']
         if not mail_id:
-            logger.error('Error: you must give an email id (--get_eml=25 by example)')
+            stop('Error: you must give an email id (--get_eml=25 by example)', logger)
         mail = handler.get_mail(mail_id)
         parsed = Parser(mail, dev_mode, mail_id)
         logger.info(parsed.headers)
         message = parsed.message
+        # structure(message)
         filename = '{}.eml'.format(mail_id)
         if arguments.get('--get_eml_orig'):
             message = parsed.initial_message
@@ -443,11 +460,47 @@ def process_mails():
         pdf_path = get_preview_pdf_path(config, mail_id.encode('utf8'))
         logger.info('Generating {} file'.format(pdf_path))
         payload, cid_parts_used = parsed.generate_pdf(pdf_path)
-        attachments = parsed.attachments(True, cid_parts_used)
-        m_at = modify_attachments(mail_id, attachments)
         handler.disconnect()
         lock.close()
         sys.exit()
+    elif arguments.get("--reset_flags"):
+        mail_id = arguments['--reset_flags']
+        if not mail_id:
+            stop('Error: you must give an email id (--reset_flags=25 by example)', logger)
+        handler.mark_reset_all(mail_id)
+        handler.disconnect()
+        lock.close()
+        sys.exit()
+    elif arguments.get("--test_eml"):
+        handler.disconnect()
+        eml_path = arguments['--test_eml']
+        if not eml_path or not os.path.exists(eml_path):
+            stop("Error: you must give an existing eml path '{}' (--test_eml=123.eml by example)".format(eml_path),
+                 logger)
+        if not dev_mode:
+            stop('Error: You must activate dev mode to test an eml file', logger)
+        with open(eml_path) as fp:
+            mail = email.message_from_file(fp, policy=email_policy)
+        mail_id = os.path.splitext(os.path.basename(eml_path))[0]
+        mail.__setitem__("X-Forwarded-For", '0.0.0.0')  # to be considered as main mail
+        parser = Parser(mail, dev_mode, '')
+        headers = parser.headers
+        main_file_path = get_preview_pdf_path(config, mail_id)
+        cid_parts_used = set()
+        try:
+            payload, cid_parts_used = parser.generate_pdf(main_file_path)
+            pdf_gen = True
+        except Exception as pdf_exc:
+            main_file_path = main_file_path.replace('.pdf', '.eml')
+            save_as_eml(main_file_path, parser.message)
+            pdf_gen = False
+        # structure(mail)
+        o_attachments = parser.attachments(pdf_gen, cid_parts_used)
+        # [{tup[0]: tup[1] for tup in at.items() if tup[0] != 'content'} for at in o_attachments]
+        attachments = modify_attachments(mail_id, o_attachments)
+        send_to_ws(config, headers, main_file_path, attachments, mail_id)
+        lock.close()
+        sys.exit()
     elif arguments.get("--stats"):
         logger.info('Started at {}'.format(datetime.now()))
         stats = handler.stats()
@@ -459,29 +512,36 @@ def process_mails():
         logger.info('Ended at {}'.format(datetime.now()))
         sys.exit()
 
-    imported = errors = unsupported = ignored = 0
+    imported = errors = unsupported = ignored = total = 0
     for mail_info in handler.get_waiting_emails():
+        total += 1
         mail_id = mail_info.id
         mail = mail_info.mail
         main_file_path = get_preview_pdf_path(config, mail_id)
         try:
             parser = Parser(mail, dev_mode, mail_id)
+            headers = parser.headers
             if parser.origin == 'Generic inbox':
-                mail_sender = parser.headers["From"][0][1]
-                notify_unsupported_origin(config, mail, mail_sender)
+                mail_sender = headers["From"][0][1]
                 if not dev_mode:
                     handler.mark_mail_as_unsupported(mail_id)
                 unsupported += 1
+                try:
+                    notify_unsupported_origin(config, mail, mail_sender)
+                except Exception:  # better to continue than advise user
+                    pass
                 continue
-            headers = parser.headers
             # we check if the pushing agent has a permitted email format
             if 'Agent' in headers and not check_transferer(headers['Agent'][0][1],
                                                            config['mailinfos'].get('sender-pattern', '.+')):
                 if not dev_mode:
                     handler.mark_mail_as_ignored(mail_id)
-                notify_ignored(config, mail_id, mail, headers['Agent'][0][1])
                 # logger.error('Rejecting {}: {}'.format(headers['Agent'][0][1], headers['Subject']))
                 ignored += 1
+                try:
+                    notify_ignored(config, mail_id, mail, headers['Agent'][0][1])
+                except Exception:  # better to continue than advise user
+                    pass
                 continue
             # logger.info('Accepting {}: {}'.format(headers['Agent'][0][1], headers['Subject']))
             cid_parts_used = set()
@@ -506,8 +566,11 @@ def process_mails():
                 handler.mark_mail_as_error(mail_id)
             errors += 1
 
-    logger.info("{} emails have been imported. {} emails are unsupported. {} emails have caused an error. {} emails "
-                "are ignored".format(imported, unsupported, errors, ignored))
+    if total:
+        logger.info("Treated {} emails: {} imported. {} unsupported. {} in error. {} ignored.".format(total,
+                    imported, unsupported, errors, ignored))
+    else:
+        logger.info("Treated no email.")
     handler.disconnect()
     lock.close()
 
@@ -572,8 +635,8 @@ def clean_mails():
         if doit:
             res, data = handler.connection.expunge()
             if res != "OK":
-                out.append(u"ERROR: Unable to deleted mails !!")
-                logger.error("Unable to deleted mails")
+                out.append(u"ERROR: Unable to delete mails !!")
+                logger.error("Unable to delete mails")
     handler.disconnect()
     out.append(u"{} emails have been deleted. {} emails are ignored. {} emails have caused an error.".format(
                deleted, ignored, error))
